# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-03-20 20:00
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='CacheEntry',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('service', models.CharField(db_index=True, max_length=50)),
                ('url', models.TextField()),
                ('url_key', models.SlugField(max_length=40, unique=True)),
                ('status', models.PositiveIntegerField()),
                ('header_pickle', models.TextField()),
                ('content', models.TextField()),
            ],
            options={
                'db_table': 'restclients_cacheentry',
            },
        ),
        migrations.CreateModel(
            name='CacheEntryTimed',
            fields=[
                ('cacheentry_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='rc_django.CacheEntry')),
                ('time_saved', models.DateTimeField()),
            ],
            options={
                'db_table': 'restclients_cacheentrytimed',
            },
            bases=('rc_django.cacheentry',),
        ),
        migrations.AlterUniqueTogether(
            name='cacheentry',
            unique_together=set([('service', 'url_key')]),
        ),
    ]
