<div><label for="college_abbreviation">College Abbreviation:</label> <input class="college_abbreviation" id="college_abbreviation" name="college_abbreviation" value="{{search.college_abbreviation}}" type="text" size="8"></div>
