<div><label for="curriculum_abbreviation">Curriculum Abbreviation:</label> <input class="curriculum_abbreviation" id="curriculum_abbreviation" name="curriculum_abbreviation" value="{{search.curriculum_abbreviation}}" type="text" size="8">&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
<label for="course_number">Course number:</label> <input class="course_number" id="course_number" name="course_number" value="{{search.course_number}}" type="text" size="3"></div>
